#  Created by Ming Li and Yuehua Cui on 3/20/2010.
#  Copyright 2010 Michigan State University. All rights reserved.

#  This code is designed for the paper in Li et al. (2010) submitted to BMC Genetics
#  Mapping haplotype-haplotype interactions with addaptive LASSO

###Estep: Given pai, SNPtype and prior of double heterzyous, calculated the mean of double heterzygous
###x:4SNP genotype y:phenotype
###X1:non-miss,X2-X4:block 1 miss X5-X7:block 2 miss X8:both block miss
doEstep<-function(x,y,beta,theda){
	omega<-matrix(1,nrow=nrow(x),ncol=2);
		A<-matrix(c(1, 1,-1/2, 1,-1/2, 1,-1/2,-1/2, 1/4,
	   		    1, 1,-1/2, 0, 1/2, 0, 1/2,   0,-1/4,
	    		    1, 1,-1/2,-1,-1/2,-1,-1/2, 1/2, 1/4,
	    		    1, 0, 1/2, 1,-1/2, 0,   0, 1/2,-1/4,
	    		    1, 0, 1/2, 0, 1/2, 0,   0,   0, 1/4,
	    		    1, 0, 1/2,-1,-1/2, 0,   0,-1/2,-1/4,
	    		    1,-1,-1/2, 1,-1/2,-1, 1/2,-1/2, 1/4,
	    		    1,-1,-1/2, 0, 1/2, 0,-1/2,   0,-1/4,
	    		    1,-1,-1/2,-1,-1/2, 1, 1/2, 1/2, 1/4),ncol=9,byrow=T);
######Neither of the blocks are heterzygous	
	pos<-(x[,1]!=12 | x[,2]!= 12)&(x[,3]!=12 | x[,4]!=12);
	B<-cbind(getBlockType(x[pos,1:2]),getBlockType(x[pos,3:4]));
	D<-getdesign.nonmiss(B);
	D.nonmiss<-cbind(D,x[pos,-1:-4]);
	D.nonmiss<-list(D.nonmiss,pos);
	names(D.nonmiss)<-c('D.nonmiss','pos');	
	result<-list(D.nonmiss=D.nonmiss);

######Block1 is double heterzygous, Block2 is not
	pos<-(x[,1]==12 & x[,2]==12 & x[,3]==11 & x[,4]==11);
	if(sum(pos)>0){
		D1<-cbind(matrix(rep(A[4,],sum(pos)),byrow=T,ncol=9),matrix(x[pos,-1:-4],nrow=sum(pos)));
		D2<-cbind(matrix(rep(A[7,],sum(pos)),byrow=T,ncol=9),matrix(x[pos,-1:-4],nrow=sum(pos)));
		pai1<-logist(D1%*%beta);
		pai2<-logist(D2%*%beta);
		a<-theda[1]*pai1    /(theda[1]*pai1    +(1-theda[1])*pai2); 
		b<-theda[1]*(1-pai1)/(theda[1]*(1-pai1)+(1-theda[1])*(1-pai2)); 
		omega[pos&(y== 1),1]<-a[y[pos]== 1]; 
		omega[pos&(y==-1),1]<-b[y[pos]==-1];
		D11<-list(D1,D2,pos);
		names(D11)<-c('D1','D2','pos');
		result<-c(result,list(D11=D11));
	}
	

	pos<-(x[,1]==12 & x[,2]==12 & ((x[,3]==11 & x[,4]==12) | (x[,3]==12 & x[,4]==11)));
	if(sum(pos)>0){
		D1<-cbind(matrix(rep(A[5,],sum(pos)),byrow=T,ncol=9),matrix(x[pos,-1:-4],nrow=sum(pos)));
		D2<-cbind(matrix(rep(A[8,],sum(pos)),byrow=T,ncol=9),matrix(x[pos,-1:-4],nrow=sum(pos)));
		pai1<-logist(D1%*%beta);
		pai2<-logist(D2%*%beta);
		a<-theda[1]*(pai1)  /(theda[1]*(pai1)  +(1-theda[1])*(pai2)); 
		b<-theda[1]*(1-pai1)/(theda[1]*(1-pai1)+(1-theda[1])*(1-pai2));
		omega[pos&(y== 1), 1]<-a[y[pos]== 1]; 
		omega[pos&(y==-1), 1]<-b[y[pos]==-1];
		D12<-list(D1,D2,pos);
		names(D12)<-c('D1','D2','pos');
		result<-c(result,list(D12=D12));
	}

	pos<-(x[,1]==12 & x[,2]==12 & ((x[,3]==11 & x[,4]==22) | (x[,3]==12 & x[,4]==22) | (x[,3]==22 & x[,4]==11) |(x[,3]==22 & x[,4]==12) |(x[,3]==22 & x[,4]==22) ));
	if(sum(pos)>0){
		D1<-cbind(matrix(rep(A[6,],sum(pos)),byrow=T,ncol=9),matrix(x[pos,-1:-4],nrow=sum(pos)));
		D2<-cbind(matrix(rep(A[9,],sum(pos)),byrow=T,ncol=9),matrix(x[pos,-1:-4],nrow=sum(pos)));
		pai1<-logist(D1%*%beta);
		pai2<-logist(D2%*%beta);
		a<-theda[1]*(pai1)  /(theda[1]*(pai1)  +(1-theda[1])*(pai2)); 
		b<-theda[1]*(1-pai1)/(theda[1]*(1-pai1)+(1-theda[1])*(1-pai2));
		omega[pos&(y== 1), 1]<-a[y[pos]== 1]; 
		omega[pos&(y==-1), 1]<-b[y[pos]==-1];
		D13<-list(D1,D2,pos);
		names(D13)<-c('D1','D2','pos');;
		result<-c(result,list(D13=D13));
	}

######Block 2 is double heterzygous, block1 is not.
	pos<-(x[,3]==12 & x[,4]==12 & x[,1]==11 & x[,2]==11);
	if(sum(pos)>0){
		D1<-cbind(matrix(rep(A[2,],sum(pos)),byrow=T,ncol=9),matrix(x[pos,-1:-4],nrow=sum(pos)));
		D2<-cbind(matrix(rep(A[3,],sum(pos)),byrow=T,ncol=9),matrix(x[pos,-1:-4],nrow=sum(pos)));
		pai1<-logist(D1%*%beta);
		pai2<-logist(D2%*%beta);
		a<-theda[2]*(pai1)  /(theda[2]*(pai1)  +(1-theda[2])*(pai2)); 
		b<-theda[2]*(1-pai1)/(theda[2]*(1-pai1)+(1-theda[2])*(1-pai2));
		omega[pos&(y== 1),2]<-a[y[pos]== 1]; 
		omega[pos&(y==-1),2]<-b[y[pos]==-1];
		D21<-list(D1,D2,pos);
		names(D21)<-c('D1','D2','pos');
		result<-c(result,list(D21=D21));
	}



	pos<-(x[,3]==12 & x[,4]==12 & ((x[,1]==11 & x[,2]==12) | (x[,1]==12 & x[,2]==11)));
	if(sum(pos)>0){
		D1<-cbind(matrix(rep(A[5,],sum(pos)),byrow=T,ncol=9),matrix(x[pos,-1:-4],nrow=sum(pos)));
		D2<-cbind(matrix(rep(A[6,],sum(pos)),byrow=T,ncol=9),matrix(x[pos,-1:-4],nrow=sum(pos)));
		pai1<-logist(D1%*%beta);
		pai2<-logist(D2%*%beta);
		a<-theda[2]*(pai1)  /(theda[2]*(pai1)  +(1-theda[2])*(pai2)); 
		b<-theda[2]*(1-pai1)/(theda[2]*(1-pai1)+(1-theda[2])*(1-pai2));
		omega[pos&(y== 1),2]<-a[y[pos]== 1]; 
		omega[pos&(y==-1),2]<-b[y[pos]==-1];
		D22<-list(D1,D2,pos);
		names(D22)<-c('D1','D2','pos');
		result<-c(result,list(D22=D22));
	}


	pos<-(x[,3]==12 & x[,4]==12 & ((x[,1]==11 & x[,2]==22) | (x[,1]==12 & x[,2]==22) | (x[,1]==22 & x[,2]==11) |(x[,1]==22 & x[,2]==12) |(x[,1]==22 & x[,2]==22) ));
	if(sum(pos)>0){
		D1<-cbind(matrix(rep(A[8,],sum(pos)),byrow=T,ncol=9),matrix(x[pos,-1:-4],nrow=sum(pos)));
		D2<-cbind(matrix(rep(A[9,],sum(pos)),byrow=T,ncol=9),matrix(x[pos,-1:-4],nrow=sum(pos)));
		pai1<-logist(D1%*%beta);
		pai2<-logist(D2%*%beta);
		a<-theda[2]*(pai1)  /(theda[2]*(pai1)  +(1-theda[2])*(pai2)); 
		b<-theda[2]*(1-pai1)/(theda[2]*(1-pai1)+(1-theda[2])*(1-pai2));
		omega[pos&(y== 1),2]<-a[y[pos]== 1]; 
		omega[pos&(y==-1),2]<-b[y[pos]==-1];
		D23<-list(D1,D2,pos);
		names(D23)<-c('D1','D2','pos');
		result<-c(result,list(D23=D23));
	}

######Both blocks are heterzygous.
	pos<-(x[,1]==12 & x[,2]==12 & x[,3]==12 & x[,4]==12);
	if(sum(pos)>0){
		D1<-cbind(matrix(rep(A[5,],sum(pos)),byrow=T,ncol=9),matrix(x[pos,-1:-4],nrow=sum(pos)));
		D2<-cbind(matrix(rep(A[6,],sum(pos)),byrow=T,ncol=9),matrix(x[pos,-1:-4],nrow=sum(pos)));
		D3<-cbind(matrix(rep(A[8,],sum(pos)),byrow=T,ncol=9),matrix(x[pos,-1:-4],nrow=sum(pos)));
		D4<-cbind(matrix(rep(A[9,],sum(pos)),byrow=T,ncol=9),matrix(x[pos,-1:-4],nrow=sum(pos)));
		pai1<-logist(D1%*%beta);
		pai2<-logist(D2%*%beta);
		pai3<-logist(D3%*%beta);
		pai4<-logist(D4%*%beta);
	
		a1<-(theda[1]*theda[2]*    pai1+theda[1]*(1-theda[2])*pai2)    /(theda[1]*theda[2]*pai1    +theda[1]*(1-theda[2])*pai2    +(1-theda[1])*theda[2]*pai3    +(1-theda[1])*(1-theda[2])*pai4); 
		b1<-(theda[1]*theda[2]*(1-pai1)+theda[1]*(1-theda[2])*(1-pai2))/(theda[1]*theda[2]*(1-pai1)+theda[1]*(1-theda[2])*(1-pai2)+(1-theda[1])*theda[2]*(1-pai3)+(1-theda[1])*(1-theda[2])*(1-pai4)); 
		a2<-(theda[1]*theda[2]*    pai1+theda[2]*(1-theda[1])*pai3)    /(theda[1]*theda[2]*pai1    +theda[1]*(1-theda[2])*pai2    +(1-theda[1])*theda[2]*pai3    +(1-theda[1])*(1-theda[2])*pai4); 
		b2<-(theda[1]*theda[2]*(1-pai1)+theda[2]*(1-theda[1])*(1-pai3))/(theda[1]*theda[2]*(1-pai1)+theda[1]*(1-theda[2])*(1-pai2)+(1-theda[1])*theda[2]*(1-pai3)+(1-theda[1])*(1-theda[2])*(1-pai4)); 	
		omega[pos&(y== 1),1]<-a1[y[pos]== 1];
		omega[pos&(y==-1),1]<-b1[y[pos]==-1];
		omega[pos&(y== 1),2]<-a2[y[pos]== 1];
		omega[pos&(y==-1),2]<-b2[y[pos]==-1];
		D.both<-list(D1,D2,D3,D4,pos);
		names(D.both)<-c('D1','D2','D3','D4','pos');
		result<-c(result,list(D.both=D.both));
	}
	result<-c(list(omega=omega),result);
	result;
}





######Neither of the blocks are heterzygous	

getDerv.nonmiss<-function(D,y,beta){

lik<-getlik(D,y,beta);
Q.derv<-D*(1-lik)*y;
F<-colSums(Q.derv);
result<-list(F,lik);
names(result)<-c('Derv1','liklihood');
result;
}

###### 12/12 & 11/11

getDerv.11<-function(D1,D2,y,beta,omega){

lik1<-getlik(D1,y,beta);
lik2<-getlik(D2,y,beta);
lik<-(omega[,1]*lik1+(1-omega[,1])*lik2);
Q.derv<-omega[,1]*D1*(1-lik1)*y+(1-omega[,1])*D2*(1-lik2)*y;
F<-colSums(Q.derv);
result<-list(F,lik);
names(result)<-c('Derv1','liklihood');
result;
}

###### 12/12 & ( 11/12 | 12/11 )

getDerv.12<-function(D1,D2,y,beta,omega){
	
lik1<-getlik(D1,y,beta);
lik2<-getlik(D2,y,beta);
lik<-(omega[,1]*lik1+(1-omega[,1])*lik2);
Q.derv<-omega[,1]*D1*(1-lik1)*y+(1-omega[,1])*D2*(1-lik2)*y;
F<-colSums(Q.derv);
result<-list(F,lik);
names(result)<-c('Derv1','liklihood');
result;
}

###### 12/12 & ( 11/22 | 12/22 | 22/11 | 22/12 | 22/22 )

getDerv.13<-function(D1,D2,y,beta,omega){
	
lik1<-getlik(D1,y,beta);
lik2<-getlik(D2,y,beta);
lik<-(omega[,1]*lik1+(1-omega[,1])*lik2);
Q.derv<-omega[,1]*D1*(1-lik1)*y+(1-omega[,1])*D2*(1-lik2)*y;
F<-colSums(Q.derv);
result<-list(F,lik);
names(result)<-c('Derv1','liklihood');
result;
}


getDerv.21<-function(D1,D2,y,beta,omega){

###### 11/11 & 12/12

lik1<-getlik(D1,y,beta);
lik2<-getlik(D2,y,beta);
lik<-(omega[,1]*lik1+(1-omega[,1])*lik2);
Q.derv<-omega[,1]*D1*(1-lik1)*y+(1-omega[,1])*D2*(1-lik2)*y;
F<-colSums(Q.derv);
result<-list(F,lik);
names(result)<-c('Derv1','liklihood');
result;
}

getDerv.22<-function(D1,D2,y,beta,omega){

######  ( 11/12 | 12/11 ) & 12/12
	
lik1<-getlik(D1,y,beta);
lik2<-getlik(D2,y,beta);
lik<-(omega[,1]*lik1+(1-omega[,1])*lik2);
Q.derv<-omega[,1]*D1*(1-lik1)*y+(1-omega[,1])*D2*(1-lik2)*y;
F<-colSums(Q.derv);
result<-list(F,lik);
names(result)<-c('Derv1','liklihood');
result;
}

getDerv.23<-function(D1,D2,y,beta,omega){

######  ( 11/22 | 12/22 | 22/11 | 22/12 | 22/22 ) & 12/12 

lik1<-getlik(D1,y,beta);
lik2<-getlik(D2,y,beta);
lik<-(omega[,1]*lik1+(1-omega[,1])*lik2);
Q.derv<-omega[,1]*D1*(1-lik1)*y+(1-omega[,1])*D2*(1-lik2)*y;
F<-colSums(Q.derv);
result<-list(F,lik);
names(result)<-c('Derv1','liklihood');
result;
}

getDerv.both<-function(D1,D2,D3,D4,y,beta,omega){

######  12/12 & 12/12 

lik1<-getlik(D1,y,beta);
lik2<-getlik(D2,y,beta);
lik3<-getlik(D3,y,beta);
lik4<-getlik(D4,y,beta);
lik<- (omega[,1]*omega[,2]*lik1+omega[,1]*(1-omega[,2])*lik2+(1-omega[,1])*omega[,2]*lik3+(1-omega[,1])*(1-omega[,2])*lik4);
Q.derv<-omega[,1]*omega[,2]*D1*(1-lik1)*y+omega[,1]*(1-omega[,2])*D2*(1-lik2)*y+(1-omega[,1])*omega[,2]*D3*(1-lik3)*y+(1-omega[,1])*(1-omega[,2])*D4*(1-lik4)*y;
		
F<-colSums(Q.derv);
result<-list(F,lik);
names(result)<-c('Derv1','liklihood');
result;
}

####M-step by Gauss-Seidel method

####given design matrix, phenotype , beta, calculate likelihood
getlik<-function(x,y,beta){
	lik<-exp(x%*%beta*y)/(1+exp(x%*%beta*y));
	lik[is.na(lik)]<-0.999;
	lik<-as.numeric(lik);
	lik;
}


####Given genotype and other covariate, beta, omega, calculate the first derivative.
getDerv<-function(Estep,y,beta){

lik<-rep(0,length(y));
omega<-Estep$omega;
F<-0;
if(sum(Estep$D.nonmiss$pos)>0){
	R.nonmiss<-getDerv.nonmiss(Estep$D.nonmiss$D.nonmiss,y[Estep$D.nonmiss$pos],beta);
	lik[Estep$D.nonmiss$pos]<-R.nonmiss$liklihood;
	F<-F+R.nonmiss[[1]]
}
if(sum(Estep$D11$pos)>0){
	R11<-getDerv.11(Estep$D11$D1,Estep$D11$D2,y[Estep$D11$pos],beta,matrix(omega[Estep$D11$pos,],ncol=2));
	lik[Estep$D11$pos]<-R11$liklihood;
	F<-F+R11[[1]];
}
if(sum(Estep$D12$pos)>0){
	R12<-getDerv.12(Estep$D12$D1,Estep$D12$D2,y[Estep$D12$pos],beta,matrix(omega[Estep$D12$pos,],ncol=2));
	lik[Estep$D12$pos]<-R12$liklihood;
	F<-F+R12[[1]];
}
if(sum(Estep$D13$pos)>0){
	R13<-getDerv.13(Estep$D13$D1,Estep$D13$D2,y[Estep$D13$pos],beta,matrix(omega[Estep$D13$pos,],ncol=2));
	lik[Estep$D13$pos]<-R13$liklihood;
	F<-F+R13[[1]];
}
if(sum(Estep$D21$pos)>0){
	R21<-getDerv.21(Estep$D21$D1,Estep$D21$D2,y[Estep$D21$pos],beta,matrix(omega[Estep$D21$pos,],ncol=2));
	lik[Estep$D21$pos]<-R21$liklihood;
	F<-F+R21[[1]];
}
if(sum(Estep$D22$pos)>0){
	R22<-getDerv.22(Estep$D22$D1,Estep$D22$D2,y[Estep$D22$pos],beta,matrix(omega[Estep$D22$pos,],ncol=2));
	lik[Estep$D22$pos]<-R22$liklihood;
	F<-F+R22[[1]];
}
if(sum(Estep$D23$pos)>0){
	R23<-getDerv.23(Estep$D23$D1,Estep$D23$D2,y[Estep$D23$pos],beta,matrix(omega[Estep$D23$pos,],ncol=2));
	lik[Estep$D23$pos]<-R23$liklihood;
	F<-F+R23[[1]];
}
if(sum(Estep$D.both$pos)>0){
	R.both<-getDerv.both(Estep$D.both$D1,Estep$D.both$D2,Estep$D.both$D3,Estep$D.both$D4,y[Estep$D.both$pos],beta,matrix(omega[Estep$D.both$pos,],ncol=2));
	lik[Estep$D.both$pos]<-R.both$liklihood;
	F<-F+R.both[[1]];
}	
result<-list(F,lik);
names(result)<-c('1stderv','liklihood');
result;
}

getVilo<-function(F,beta,lamda,weights=rep(1,length(beta))){
	V<-rep(0,length(F));
	V[beta>0]<-abs(lamda*weights-F)[beta>0];
	V[beta<0]<-abs(lamda*weights+F)[beta<0];
	V[beta==0]<-apply(cbind(F-lamda*weights,-lamda*weights-F,0),1,FUN=max)[beta==0];
	V[1]<-abs(F[1]);
	V;
}

GS_Mstep<-function(Estep,y,beta,lamda=10,weights=rep(1,length(beta)),stopvalue=10^(-5),iteration=50){
	b<-beta;
	Derv<-getDerv(Estep,y,beta);
	F<-Derv[[1]];
	V<-getVilo(F,b,lamda,weights);
	z <-b==0;
	nz<-b!=0;
	it<-0;
	while(sum(V[z]>stopvalue)>0){
		v<- (V==max(V[z]))&z;
		if(sum(v)>1){
			shit<-order(v,decreasing=T)[1:(sum(v)-1)];
			v[shit]<-(1<0);
		}	
		b[v]<-getUpdate(Estep,y,b,v,lamda,weights=weights,stopvalue=stopvalue);
		Derv<-getDerv(Estep,y,b);
		F<-Derv[[1]];
		V<-getVilo(F,b,lamda,weights);
		z <-b==0;
		nz<-b!=0;
		it1<-0;
		while(sum(V[nz]>stopvalue)>0){
			v<- (V==max(V[nz]))& nz;	
			if(sum(v)>1){
				shit<-order(v,decreasing=T)[1:(sum(v)-1)];
				v[shit]<-(1<0);
			}	
			b[v]<-getUpdate(Estep,y,b,v,lamda,weights=weights,stopvalue=stopvalue);	
			Derv<-getDerv(Estep,y,b);
			F<-Derv[[1]];
			V<-getVilo(F,b,lamda,weights);
			z <-b==0;
			nz<-b!=0;		
			it1<-it1+1;
			if(it1>iteration){
				break;
			}
		}	
		it<-it+1;
		if(it>iteration){
			break;
		}		
	}
	AIC<- -2*sum(log(Derv[[2]]))+ sum(beta[-1]!=0)*2;
	BIC<- -2*sum(log(Derv[[2]]))+ sum(beta[-1]!=0)*log(length(y));
	list(b,AIC,BIC);
} 

####Update one beta[i] only, while given other beta[j].


getUpdate<-function(Estep,y,beta,v,lamda,weights=rep(1,length(beta)),stopvalue=10^(-5),iteration=50){
	b0<-beta;
	b1<-beta;
	b2<-beta;
	b0[v]<-0;
	Derv0<-getDerv(Estep,y,b0);
	f0<-Derv0[[1]][v];
	Derv<-getDerv(Estep,y,beta);
	f<-Derv[[1]][v];
	step<-2;
	f1<-f;
	f2<-f;

######update intercept

	if(order(v,decreasing=T)[1]==1){
		while(f*f1>0 & f*f2>0){
			b1[v]<-beta[v]+step;
			b2[v]<-beta[v]-step;
			Derv1<-getDerv(Estep,y,b1);
			f1<-Derv1[[1]][v];
			Derv2<-getDerv(Estep,y,b2);
			f2<-Derv2[[1]][v];
		}
		if(f*f1<0){
			L<-beta[v];
			H<-b1[v];
			fL<-f;
			fH<-f1;
		}
		if(f*f2<0){
			L<-b2[v];
			H<-beta[v];
			fL<-f2;
			fH<-f;
		}
		b<-beta;
		b[v]<-(L+H)/2;
		Derv<-getDerv(Estep,y,b);
		f<-Derv[[1]][v];
		while(abs(f)>stopvalue){
			if(f*fL<0){
				H<-(L+H)/2;
				fH<-f;
				b[v]<-(L+H)/2;
				Derv<-getDerv(Estep,y,b);
				f<-Derv[[1]][v];
			}
			if(f*fH<0){
				L<-(L+H)/2;
				fL<-f;
				b[v]<-(L+H)/2;
				Derv<-getDerv(Estep,y,b);
				f<-Derv[[1]][v];
			}	
		}
	}
	

#### Non-intercept coefficient

	if(order(v,decreasing=T)[1]>1){
		if(f0<=lamda*weights[v] & f0>=(-lamda*weights[v])){
			b<-beta;
			b[v]<-0;
		}
		if(f0>lamda*weights[v]){
			b1[v]<-0;
			Derv1<-getDerv(Estep,y,b1);
			f1<-Derv1[[1]][v];
			b2[v]<-b1[v]+step;
			Derv2<-getDerv(Estep,y,b2);
			f2<-Derv2[[1]][v];
			while(f2>lamda*weights[v]){
				b1[v]<-b2[v];
				b2[v]<-b2[v]+step;
				f1<-f2;
				Derv2<-getDerv(Estep,y,b2);
				f2<-Derv2[[1]][v];
			}
			L<-b1[v];
			H<-b2[v];			

#######Bi-section update
			b<-beta;
			b[v]<-(L+H)/2;
			Derv<-getDerv(Estep,y,b);
			f<-Derv[[1]][v];
			it<-0;
			while(abs(f-lamda*weights[v])>stopvalue){
				if((f-lamda*weights[v])<0){
					H<-(L+H)/2;
					fH<-f;
					b[v]<-(L+H)/2;
					Derv<-getDerv(Estep,y,b);
					f<-Derv[[1]][v];
				}
				if((f-lamda*weights[v])>0){
					L<-(L+H)/2;
					fL<-f;
					b[v]<-(L+H)/2;
					Derv<-getDerv(Estep,y,b);
					f<-Derv[[1]][v];
				}	
				it<-it+1;
				if(it>iteration){
					break;
				}
			}
		}
		if(f0< -lamda*weights[v]){
			b1[v]<-0;
			Derv1<-getDerv(Estep,y,b1);
			f1<-Derv1[[1]][v];
			b2[v]<-b1[v]-step;
			Derv2<-getDerv(Estep,y,b2);
			f2<-Derv2[[1]][v];
			while(f2< -lamda*weights[v]){
				b1[v]<-b2[v];
				b2[v]<-b2[v]-step;
				f1<-f2;
				Derv2<-getDerv(Estep,y,b2);
				f2<-Derv2[[1]][v];
			}
			L<-b2[v];
			H<-b1[v];
			
######Bi-section update
			it<-0;
			b<-beta;
			b[v]<-(L+H)/2;
			Derv<-getDerv(Estep,y,b);
			f<-Derv[[1]][v];
			while(abs(f+lamda*weights[v])>stopvalue){
				if((f+lamda*weights[v])<0){
					H<-(L+H)/2;
					fH<-f;
					b[v]<-(L+H)/2;
					Derv<-getDerv(Estep,y,b);
					f<-Derv[[1]][v];
				}
				if((f+lamda*weights[v])>0){
					L<-(L+H)/2;
					fL<-f;
					b[v]<-(L+H)/2;
					Derv<-getDerv(Estep,y,b);
					f<-Derv[[1]][v];
				}	
				it<-it+1;
				if(it>iteration){
					 break;
				}
			}		
		}
	}				
b[v];
}





#Two block interaction


#Code the SNP type.
#x:observed SNPs type.
getSNPtype<-function(x){
	y<-matrix(0,nrow=nrow(x),ncol=ncol(x));
	k<-ncol(x);
	for(i in 1:k){
		snp<-names(table(x[,i]));
		y[x[,i]==snp[1],i]<-11;
		y[x[,i]==snp[2],i]<-12;
		y[x[,i]==snp[3],i]<-22;
	}
	y;
} 

#Get disign matrix from block type. x:genotype of block 1, y: genotype of block 2
getdesign.nonmiss<-function(x){
	z<-matrix(0,ncol=nrow(x),nrow=9);
	z[,x[,1]== 1 & x[,2]== 1]<- c( 1, 1,-1/2, 1,-1/2, 1,-1/2,-1/2, 1/4);
	z[,x[,1]== 1 & x[,2]== 0]<- c( 1, 1,-1/2, 0, 1/2, 0, 1/2,   0,-1/4);
	z[,x[,1]== 1 & x[,2]==-1]<- c( 1, 1,-1/2,-1,-1/2,-1,-1/2, 1/2, 1/4);
	z[,x[,1]== 0 & x[,2]== 1]<- c( 1, 0, 1/2, 1,-1/2, 0,   0, 1/2,-1/4);
	z[,x[,1]== 0 & x[,2]== 0]<- c( 1, 0, 1/2, 0, 1/2, 0,   0,   0, 1/4);
	z[,x[,1]== 0 & x[,2]==-1]<- c( 1, 0, 1/2,-1,-1/2, 0,   0,-1/2,-1/4);
	z[,x[,1]==-1 & x[,2]== 1]<- c( 1,-1,-1/2, 1,-1/2,-1, 1/2,-1/2, 1/4);
	z[,x[,1]==-1 & x[,2]== 0]<- c( 1,-1,-1/2, 0, 1/2, 0,-1/2,   0,-1/4);
	z[,x[,1]==-1 & x[,2]==-1]<- c( 1,-1,-1/2,-1,-1/2, 1, 1/2, 1/2, 1/4);
	z<-aperm(z);
	colnames(z)<-c('mu','a1','d1','a2','d2','a1a2','a1d2','a2d1','d1d2');
	z;
}

#Given 2SNP block genotype,estimate frequency [11][22] in double heterzyger 11/22. x: 2SNP type
getRelativeFreq<-function(x){
	n<-matrix(0,nrow=3,ncol=3);
	n[1,1]<-sum((x[,1]=='11')*(x[,2]=='11'));
	n[1,2]<-sum((x[,1]=='11')*(x[,2]=='12'));
	n[1,3]<-sum((x[,1]=='11')*(x[,2]=='22'));
	n[2,1]<-sum((x[,1]=='12')*(x[,2]=='11'));
	n[2,2]<-sum((x[,1]=='12')*(x[,2]=='12'));
	n[2,3]<-sum((x[,1]=='12')*(x[,2]=='22'));
	n[3,1]<-sum((x[,1]=='22')*(x[,2]=='11'));
	n[3,2]<-sum((x[,1]=='22')*(x[,2]=='12'));
	n[3,3]<-sum((x[,1]=='22')*(x[,2]=='22'));
	theda<-0;
	update<-0.5;
	while(abs(theda-update)>10^(-7)){
		theda<-update;
		p<-c(2*n[1,1]+n[1,2]+n[2,1]+theda*n[2,2],n[1,2]+2*n[1,3]+(1-theda)*n[2,2]+n[2,3],
				n[2,1]+(1-theda)*n[2,2]+2*n[3,1]+n[3,2],theda*n[2,2]+n[2,3]+n[3,2]+2*n[3,3])/sum(n)/2;
		update<-(p[1]*p[4])/(p[1]*p[4]+p[2]*p[3]);
	}
	theda;
}

#Given the genotype of non-double heterzygous genotype, get the block type. x:2SNP genotype

getBlockType<-function(x){
	y<-rep(-1,nrow(x));
	y[x[,1]==11 & x[,2]==11] <- 1;
	y[x[,1]==11 & x[,2]==12] <- 0;
	y[x[,1]==12 & x[,2]==11] <- 0;
	y;
} 

#Given Block type, get hapmap type x:Block type
gethaptype<-function(x){
	y<-rep(-1,nrow(x));
	y[x[,1]== 1 & x[,2]== 1]<- 1;
	y[x[,1]== 1 & x[,2]== 0]<- 2;
	y[x[,1]== 1 & x[,2]==-1]<- 3;
	y[x[,1]== 0 & x[,2]== 1]<- 4;
	y[x[,1]== 0 & x[,2]== 0]<- 5;
	y[x[,1]== 0 & x[,2]==-1]<- 6;
	y[x[,1]==-1 & x[,2]== 1]<- 7;
	y[x[,1]==-1 & x[,2]== 0]<- 8;
	y[x[,1]==-1 & x[,2]==-1]<- 9;
	y;
}

#logist function
logist<-function(x){
	y<-exp(x)/(1+exp(x));
	y[is.na(y)]<-1;
	y<-as.numeric(y);
	y[y==1]<-0.99;
	y[y==0]<-0.01;
	y;
}



#####function for simulation
####generate 1 copy of haplotype
gethaplotype<-function(n=500,pA=0.7,pB=0.7,D=0.02){
	haplotype<-matrix(0,nrow=n,ncol=2);
	sample<-runif(n,min=0,max=1);
	pAB<-pA*pB+D;
	pAb<-pA*(1-pB)-D;
	paB<-(1-pA)*pB-D;
	pab<-(1-pA)*(1-pB)+D;
	haplotype[sample<=pAB,1]<-1;
	haplotype[sample<=pAB,2]<-1;
	haplotype[sample>pAB & sample<=pA,1]<-1;
	haplotype[sample>pAB & sample<=pA,2]<-2;
	haplotype[sample>pA  & sample<=pA+paB,1]<-2;
	haplotype[sample>pA  & sample<=pA+paB,2]<-1;
	haplotype[sample>pA+paB,1]<-2;
	haplotype[sample>pA+paB,2]<-2;
	haplotype;
}

###given two copy haplotype (x,y), get compositediplotype and genotype
getgenotype<-function(x,y){
	genotype<-matrix(0,nrow=nrow(x),ncol=2);
	blocktype<-rep(-1,nrow(x));
	pos<-x[,1]==1 & x[,2]==1 & y[,1]==1 & y[,2]==1;
	genotype[pos,1]<-11;
	genotype[pos,2]<-11;
	blocktype[pos]<-1;
	pos<-x[,1]==1 & x[,2]==1 & y[,1]==1 & y[,2]==2;
	genotype[pos,1]<-11;
	genotype[pos,2]<-12;
	blocktype[pos]<-0;
	pos<-x[,1]==1 & x[,2]==2 & y[,1]==1 & y[,2]==1;
	genotype[pos,1]<-11;
	genotype[pos,2]<-12;
	blocktype[pos]<-0;
	pos<-x[,1]==1 & x[,2]==2 & y[,1]==1 & y[,2]==2;
	genotype[pos,1]<-11;
	genotype[pos,2]<-22;	
	pos<-x[,1]==1 & x[,2]==1 & y[,1]==2 & y[,2]==1;
	genotype[pos,1]<-12;
	genotype[pos,2]<-11;
	blocktype[pos]<-0;
	pos<-x[,1]==2 & x[,2]==1 & y[,1]==1 & y[,2]==1;
	genotype[pos,1]<-12;
	genotype[pos,2]<-11;
	blocktype[pos]<-0;
	pos<-x[,1]==1 & x[,2]==1 & y[,1]==2 & y[,2]==2;
	genotype[pos,1]<-12;
	genotype[pos,2]<-12;
	blocktype[pos]<-0;
	pos<-x[,1]==2 & x[,2]==2 & y[,1]==1 & y[,2]==1;
	genotype[pos,1]<-12;
	genotype[pos,2]<-12;
	blocktype[pos]<-0;
	pos<-x[,1]==1 & x[,2]==2 & y[,1]==2 & y[,2]==1;
	genotype[pos,1]<-12;
	genotype[pos,2]<-12;
	pos<-x[,1]==2 & x[,2]==1 & y[,1]==1 & y[,2]==2;
	genotype[pos,1]<-12;
	genotype[pos,2]<-12;
	pos<-x[,1]==1 & x[,2]==2 & y[,1]==2 & y[,2]==2;
	genotype[pos,1]<-12;
	genotype[pos,2]<-22;
	pos<-x[,1]==2 & x[,2]==2 & y[,1]==1 & y[,2]==2;
	genotype[pos,1]<-12;
	genotype[pos,2]<-22;
	pos<-x[,1]==2 & x[,2]==1 & y[,1]==2 & y[,2]==1;
	genotype[pos,1]<-22;
	genotype[pos,2]<-11;
	pos<-x[,1]==2 & x[,2]==1 & y[,1]==2 & y[,2]==2;
	genotype[pos,1]<-22;
	genotype[pos,2]<-12;
	pos<-x[,1]==2 & x[,2]==2 & y[,1]==2 & y[,2]==1;
	genotype[pos,1]<-22;
	genotype[pos,2]<-12;
	pos<-x[,1]==2 & x[,2]==2 & y[,1]==2 & y[,2]==2;
	genotype[pos,1]<-22;
	genotype[pos,2]<-22;
	z<-list(genotype,blocktype);
	names(z)<-c('genotype','blocktype');
	z;
}



####Given Block type, get design matrix

getD<-function(x){
A<-matrix(c(1, 1,-1/2, 1,-1/2, 1,-1/2,-1/2, 1/4,
	    1, 1,-1/2, 0, 1/2, 0, 1/2,   0,-1/4,
	    1, 1,-1/2,-1,-1/2,-1,-1/2, 1/2, 1/4,
	    1, 0, 1/2, 1,-1/2, 0,   0, 1/2,-1/4,
	    1, 0, 1/2, 0, 1/2, 0,   0,   0, 1/4,
	    1, 0, 1/2,-1,-1/2, 0,   0,-1/2,-1/4,
	    1,-1,-1/2, 1,-1/2,-1, 1/2,-1/2, 1/4,
	    1,-1,-1/2, 0, 1/2, 0,-1/2,   0,-1/4,
	    1,-1,-1/2,-1,-1/2, 1, 1/2, 1/2, 1/4),ncol=9,byrow=T);
y<-rep(-1,nrow(x));
y[x[,1]== 1 & x[,2]== 1]<- 1;
y[x[,1]== 1 & x[,2]== 0]<- 2;
y[x[,1]== 1 & x[,2]==-1]<- 3;
y[x[,1]== 0 & x[,2]== 1]<- 4;
y[x[,1]== 0 & x[,2]== 0]<- 5;
y[x[,1]== 0 & x[,2]==-1]<- 6;
y[x[,1]==-1 & x[,2]== 1]<- 7;
y[x[,1]==-1 & x[,2]== 0]<- 8;
y[x[,1]==-1 & x[,2]==-1]<- 9;
D<-A[y,];
D;
}

####Given design matrix and beta, get phenotype

getphenotype<-function(D,beta=c(0,0.5,0.3,0.5,0.3,0.4,0.2,0.2,0.1)){
	pai<-logist(D%*%beta);
	y<-runif(nrow(D));
	1-(y>=pai);
}

#logist function
logist<-function(x){
	y<-exp(x)/(1+exp(x));
	y[is.na(y)]<-1;
	y<-as.numeric(y);
	y[y==1]<-0.99;
	y[y==0]<-0.01;
	y;
}

glm.lasso<-function(x,y,theda,lamda=10,weights=rep(1,ncol(x)+5),stopvalue=10^(-7),iteration=50){
	beta<-1;
	update.beta<-rep(0,ncol(x)+5);
	it<-0;
	while(sum(abs(beta-update.beta))>stopvalue){
		beta<-update.beta;
		Estep<-doEstep(x,y,beta,theda);
		Mstep<-GS_Mstep(Estep,y,beta,lamda,weights);
		update.beta<-Mstep[[1]];
		AIC<-Mstep[[2]];
		BIC<-Mstep[[3]];
		it<-it+1;
		if(it>iteration){
			break;
		}
	}
	result<-list(beta,AIC,BIC);
	names(result)<-c('beta','AIC','BIC');
	result;
}

glm.lasso.BIC<-function(X,y,lamdaM,step=20,stopvalue=10^(-7),iteration=50){
	risk1<-c(11,12,21,22);
	risk2<-risk1;
	risk<-NULL;
	TEMP<-NULL;
	TEMPbic<-NULL;
	cat('Searching for optimal risk haplotype :','\n');
	cat(0,'out of 16 completed','\n');
	for(r1 in 1:4){
		for(r2 in 1:4){
	risk<-rbind(risk,c(risk1[r1],risk2[r2]));
	x<-codeSNP(X[,1:4],risk1=risk1[r1],risk2=risk2[r2]);
	x<-cbind(x,X[,-(1:4)]);		
	x<-as.matrix(x);
	geno1<-x[,1:2];
	geno2<-x[,3:4];
	theda<-c(getRelativeFreq(geno1),getRelativeFreq(geno2));
	B1<-glm.lasso(x,y,theda,lamda=0,stopvalue=10^(-7),iteration=50);
	OML<-B1$beta;
	B5<-glm.lasso(x,y,theda,lamda=lamdaM,weights=1/abs(OML),stopvalue=stopvalue,iteration)
	BICall0<-B5$BIC;
	while(sum(abs(B5$beta[-1]))>0){
		lamdaM<-lamdaM*2;
		B5<-glm.lasso(x,y,theda,lamda=lamdaM,weights=1/abs(OML),stopvalue=stopvalue,iteration=50)
	}

	lamda<-seq(0,lamdaM,length=5)
	
	B2<-glm.lasso(x,y,theda,lamda=lamda[2],weights=1/abs(OML),stopvalue=stopvalue,iteration=iteration);
	B3<-glm.lasso(x,y,theda,lamda=lamda[3],weights=1/abs(OML),stopvalue=stopvalue,iteration=iteration);
	B4<-glm.lasso(x,y,theda,lamda=lamda[4],weights=1/abs(OML),stopvalue=stopvalue,iteration=iteration);
	
	BIC<-c(B1$BIC,B2$BIC,B3$BIC,B4$BIC,B5$BIC);
	beta<-cbind(B1$beta,B2$beta,B3$beta,B4$beta,B5$beta);	

	for(i in 1:step){
		sn<-order(BIC)[1];
		if(sn>1 & sn<5){
			lamda<-seq(lamda[sn-1],lamda[sn+1],length=5)
			B2<-glm.lasso(x,y,theda,lamda=lamda[2],weights=1/abs(OML),stopvalue,iteration);
			B4<-glm.lasso(x,y,theda,lamda=lamda[4],weights=1/abs(OML),stopvalue,iteration);
			BIC<-c(BIC[sn-1],B2$BIC,BIC[sn],B4$BIC,BIC[sn+1]);
			beta<-cbind(beta[,sn-1],B2$beta,beta[,sn],B4$beta,beta[,sn+1]);	
		}
		if(sn==1){
			lamda<-seq(lamda[1],lamda[2],length=5);
			B2<-glm.lasso(x,y,theda,lamda=lamda[2],weights=1/abs(OML),stopvalue,iteration);
			B3<-glm.lasso(x,y,theda,lamda=lamda[3],weights=1/abs(OML),stopvalue,iteration);
			B4<-glm.lasso(x,y,theda,lamda=lamda[4],weights=1/abs(OML),stopvalue,iteration);
			BIC<-c(BIC[1],B2$BIC,B3$BIC,B4$BIC,BIC[2]);
			beta<-cbind(beta[,1],B2$beta,B3$beta,B4$beta,beta[,2]);	
		}
		if(sn==5){
			lamda<-seq(lamda[4],lamda[5],length=5);
			B2<-glm.lasso(x,y,theda,lamda=lamda[2],weights=1/abs(OML),stopvalue,iteration);
			B3<-glm.lasso(x,y,theda,lamda=lamda[3],weights=1/abs(OML),stopvalue,iteration);
			B4<-glm.lasso(x,y,theda,lamda=lamda[4],weights=1/abs(OML),stopvalue,iteration);
			BIC<-c(BIC[4],B2$BIC,B3$BIC,B4$BIC,BIC[5]);
			beta<-cbind(beta[,4],B2$beta,B3$beta,B4$beta,beta[,5]);	
		}

	}
	if(BIC[sn]>BICall0){
		beta[,sn]<-0;
		BIC[sn]<-BICall0;
	}
	temp<-list(beta=beta[,sn],lamda=lamda[sn],BIC=BIC[sn]);
	TEMP<-c(TEMP,list(temp));
	TEMPbic<-c(TEMPbic,BIC[sn]);
	cat(((r1-1)*4+r2),'out of 16 completed','\n');
		}	
	}
	kk<-which(TEMPbic==min(TEMPbic));
	RISK<-risk[kk,];
	out<-c(TEMP[[kk]],RiskHap=RISK);
	out;
}

glm.lasso.bic1<-function(x,y,lamdaM,step=20,stopvalue=10^(-7),iteration=50){
	x<-as.matrix(x);
	geno1<-x[,1:2];
	geno2<-x[,3:4];
	theda<-c(getRelativeFreq(geno1),getRelativeFreq(geno2));
	B1<-glm.lasso(x,y,theda,lamda=0,stopvalue=10^(-7),iteration=50);
	OML<-B1$beta;
	B5<-glm.lasso(x,y,theda,lamda=lamdaM,weights=1/abs(OML),stopvalue=stopvalue,iteration)
	BICall0<-B5$BIC;
	while(sum(abs(B5$beta[-1]))>0){
		lamdaM<-lamdaM*2;
		B5<-glm.lasso(x,y,theda,lamda=lamdaM,weights=1/abs(OML),stopvalue=stopvalue,iteration=50)
	}

	lamda<-seq(0,lamdaM,length=5)
	
	B2<-glm.lasso(x,y,theda,lamda=lamda[2],weights=1/abs(OML),stopvalue=stopvalue,iteration=iteration);
	B3<-glm.lasso(x,y,theda,lamda=lamda[3],weights=1/abs(OML),stopvalue=stopvalue,iteration=iteration);
	B4<-glm.lasso(x,y,theda,lamda=lamda[4],weights=1/abs(OML),stopvalue=stopvalue,iteration=iteration);
	
	BIC<-c(B1$BIC,B2$BIC,B3$BIC,B4$BIC,B5$BIC);
	beta<-cbind(B1$beta,B2$beta,B3$beta,B4$beta,B5$beta);	

	for(i in 1:step){
		sn<-order(BIC)[1];
		if(sn>1 & sn<5){
			lamda<-seq(lamda[sn-1],lamda[sn+1],length=5)
			B2<-glm.lasso(x,y,theda,lamda=lamda[2],weights=1/abs(OML),stopvalue,iteration);
			B4<-glm.lasso(x,y,theda,lamda=lamda[4],weights=1/abs(OML),stopvalue,iteration);
			BIC<-c(BIC[sn-1],B2$BIC,BIC[sn],B4$BIC,BIC[sn+1]);
			beta<-cbind(beta[,sn-1],B2$beta,beta[,sn],B4$beta,beta[,sn+1]);	
		}
		if(sn==1){
			lamda<-seq(lamda[1],lamda[2],length=5);
			B2<-glm.lasso(x,y,theda,lamda=lamda[2],weights=1/abs(OML),stopvalue,iteration);
			B3<-glm.lasso(x,y,theda,lamda=lamda[3],weights=1/abs(OML),stopvalue,iteration);
			B4<-glm.lasso(x,y,theda,lamda=lamda[4],weights=1/abs(OML),stopvalue,iteration);
			BIC<-c(BIC[1],B2$BIC,B3$BIC,B4$BIC,BIC[2]);
			beta<-cbind(beta[,1],B2$beta,B3$beta,B4$beta,beta[,2]);	
		}
		if(sn==5){
			lamda<-seq(lamda[4],lamda[5],length=5);
			B2<-glm.lasso(x,y,theda,lamda=lamda[2],weights=1/abs(OML),stopvalue,iteration);
			B3<-glm.lasso(x,y,theda,lamda=lamda[3],weights=1/abs(OML),stopvalue,iteration);
			B4<-glm.lasso(x,y,theda,lamda=lamda[4],weights=1/abs(OML),stopvalue,iteration);
			BIC<-c(BIC[4],B2$BIC,B3$BIC,B4$BIC,BIC[5]);
			beta<-cbind(beta[,4],B2$beta,B3$beta,B4$beta,beta[,5]);	
		}

	}
	if(BIC[sn]>BICall0){
		beta[,sn]<-0;
		BIC[sn]<-BICall0;
	}
	list(beta=beta[,sn],lamda=lamda[sn],BIC=BIC[sn]);
}




glm.lasso.AIC<-function(x,y,lamdaM,step=20,stopvalue=10^(-7),iteration=50){
	geno1<-x[,1:2];
	geno2<-x[,3:4];
	theda<-c(getRelativeFreq(geno1),getRelativeFreq(geno2));
	B1<-glm.lasso(x,y,theda,lamda=0,stopvalue=10^(-7),iteration=50);
	OML<-B1$beta;
	B5<-glm.lasso(x,y,theda,lamda=lamdaM,weights=1/abs(OML),stopvalue,iteration)
	while(sum(abs(B5$beta[-1]))>0){
		lamdaM<-lamdaM*2;
		B5<-glm.lasso(x,y,theda,lamda=lamdaM,weights=1/abs(OML),stopvalue=10^(-7),iteration=50)
	}

	lamda<-seq(0,lamdaM,length=5)
	
	B2<-glm.lasso(x,y,theda,lamda=lamda[2],weights=1/abs(OML),stopvalue,iteration);
	B3<-glm.lasso(x,y,theda,lamda=lamda[3],weights=1/abs(OML),stopvalue,iteration);
	B4<-glm.lasso(x,y,theda,lamda=lamda[4],weights=1/abs(OML),stopvalue,iteration);
	
	AIC<-c(B1$AIC,B2$AIC,B3$AIC,B4$AIC,B5$AIC);
	beta<-cbind(B1$beta,B2$beta,B3$beta,B4$beta,B5$beta);	

	for(i in 1:step){
		sn<-order(AIC)[1];
		if(sn>1 & sn<5){
			lamda<-seq(lamda[sn-1],lamda[sn+1],length=5)
			B2<-glm.lasso(x,y,theda,lamda=lamda[2],weights=1/abs(OML),stopvalue,iteration);
			B4<-glm.lasso(x,y,theda,lamda=lamda[4],weights=1/abs(OML),stopvalue,iteration);
			AIC<-c(AIC[sn-1],B2$AIC,AIC[sn],B4$AIC,AIC[sn+1]);
			beta<-cbind(beta[,sn-1],B2$beta,beta[,sn],B4$beta,beta[,sn+1]);	
		}
		if(sn==1){
			lamda<-seq(lamda[1],lamda[2],length=5);
			B2<-glm.lasso(x,y,theda,lamda=lamda[2],weights=1/abs(OML),stopvalue,iteration);
			B3<-glm.lasso(x,y,theda,lamda=lamda[3],weights=1/abs(OML),stopvalue,iteration);
			B4<-glm.lasso(x,y,theda,lamda=lamda[4],weights=1/abs(OML),stopvalue,iteration);
			AIC<-c(AIC[1],B2$AIC,B3$AIC,B4$AIC,AIC[2]);
			beta<-cbind(beta[,1],B2$beta,B3$beta,B4$beta,beta[,2]);	
		}
		if(sn==5){
			lamda<-seq(lamda[4],lamda[5],length=5);
			B2<-glm.lasso(x,y,theda,lamda=lamda[2],weights=1/abs(OML),stopvalue,iteration);
			B3<-glm.lasso(x,y,theda,lamda=lamda[3],weights=1/abs(OML),stopvalue,iteration);
			B4<-glm.lasso(x,y,theda,lamda=lamda[4],weights=1/abs(OML),stopvalue,iteration);
			AIC<-c(AIC[4],B2$AIC,B3$AIC,B4$AIC,AIC[5]);
			beta<-cbind(beta[,4],B2$beta,B3$beta,B4$beta,beta[,5]);	
		}

	}
	list(beta=beta[,sn],lamda=lamda[sn],AIC=AIC[sn]);
}



test.glm.lasso<-function(X,y,perm=500,method='BIC',lamdaM=100,step=20,stopvalue=10^(-7),iteration=50,risk){
	x<-codeSNP(X[,1:4],risk1=risk[1],risk2=risk[2]);	
	x<-as.matrix(x);
	n<-0;
	if(method=='BIC'){	
		beta<-glm.lasso.bic1(x,y,lamdaM,step=step,stopvalue=stopvalue,iteration=iteration);
		for(rep in 1: perm){
			cat('Calculating Permutation','rep','\n');
			yperm<-sample(y,length(y),replace=F);
			beta0<-glm.lasso.bic1(x,yperm,lamdaM,step=step,stopvalue=stopvalue,iteration=iteration);
			n<-n+abs(beta0$beta)>0;
		}
		p<-n/perm;	
	}
	if(method=='AIC'){	
		beta<-glm.lasso.AIC(x,y,lamdaM,step=step,stopvalue=stopvalue,iteration=iteration);
		for(rep in 1: perm){
			sn<-sample(y,length(y),replace=F);
			beta0<-glm.lasso.AIC(x,yperm,lamdaM,step=step,stopvalue=stopvalue,iteration=iteration);
			n<-n+abs(beta0$beta)>0;
		}
		p<-n/perm;	
	}
	p[beta$beta==0]<-'NA';
	p[1]<-NA;
	p;
}


codeSNP<-function(x,levels=c(0,1,2),risk1=11,risk2=11){
	X<-x;
	r11<-substring(risk1,1,1);
	r12<-substring(risk1,2,2);
	r21<-substring(risk2,1,1);
	r22<-substring(risk2,2,2);
	r<-c(r11,r12,r21,r22);
	for(i in 1:4){
		if(r[i]==1){
			X[x[,i]==levels[1],i]<-11;
			X[x[,i]==levels[2],i]<-12;
			X[x[,i]==levels[3],i]<-22;	
		}
		if(r[i]==2){
			X[x[,i]==levels[1],i]<-22;
			X[x[,i]==levels[2],i]<-12;
			X[x[,i]==levels[3],i]<-11;
		}
	}
	X;
}
 



































 






 

